//
// Centova Reseller API - https://centova.com
// Copyright 2021, Centova Technologies Inc.
//
// This library is a Go implementation of a Centova Technologies Reseller API client.
//

package resellerapi

import (
	"errors"
	"fmt"
	"net/url"
	"regexp"
	"strconv"
	"strings"

	"centova.com/resellerapi/response"
)

// Client provides an API client for the Reseller API
type Client struct {
	// the API username
	username string
	// the API key
	apikey   string
	// true for test mode, false for live
	testmode bool

	// the domain (always `centova.com`)
	domain       string
	// the endpoint base, depending on the value of testmode
	endpointBase string
}

// NewClient creates a new API client with the specified username, API key, and mode.
func NewClient(username, apikey string, testmode bool) *Client {
	baseURI := "/cp/api/v1/reseller"
	if testmode {
		baseURI = "/cp/api/v1/resellertest"
	}

	return &Client{
		username:     username,
		apikey:       apikey,
		domain:       "centova.com",
		endpointBase: baseURI,
	}
}

// SetDomain is for maintainer use only.
func (c *Client) SetDomain(domain string) {
	c.domain = domain
}

var (
	// ErrInvalidArgument is returned when an invalid argument is passed to an API function
	ErrInvalidArgument = errors.New("invalid argument")

	// reDomain performs (very) basic validation on a domain name
	reDomain = regexp.MustCompile("^.+\\..+$")
	// reEmail performs (very) basic validation on an email address
	reEmail  = regexp.MustCompile("^.+@.+\\..+$")
)

// OrderLicense places an order for the specified package.
//   packageid    The ID number of the package to order (see the *Package List* endpoint)
//   period       The renewal period for the license
//   domain       The domain name for the new license
//   email        The end-user's email address (used to email license key to user)
//   installation true if installation service is being ordered, otherwise false
//   customident  Any custom identifier that you want to associate with thislicense
//   coupon       A coupon code to apply to this purchase (will negate any reseller discount)
func (c *Client) OrderLicense(packageid int, period BillingPeriod, domain string, email string, installation bool, customident string, coupon string) (*response.Order, error) {
	if !reDomain.MatchString(domain) {
		return nil, fmt.Errorf("%w: bad domain name", ErrInvalidArgument)
	}
	if !reEmail.MatchString(email) {
		return nil, fmt.Errorf("%w: bad email", ErrInvalidArgument)
	}
	if packageid < 1 {
		return nil, fmt.Errorf("%w: bad package ID", ErrInvalidArgument)
	}

	inst := "0"
	if installation {
		inst = "1"
	}

	args := url.Values{}
	args.Add("package", strconv.FormatInt(int64(packageid), 10))
	args.Add("period", billingPeriods[period])
	args.Add("domain", domain)
	args.Add("email", email)
	args.Add("installation", inst)
	args.Add("customid", customident)
	if len(coupon) > 0 {
		args.Add("coupon", coupon)
	}

	req := NewRequest(c, "order", &args)
	resp, err := req.Dispatch()
	if err != nil {
		return nil, err
	}

	r := &response.Order{}
	return r, resp.UnmarshalData(r)
}

// CancelLicense cancels an active license and terminates recurring billing (if any).
//   licenseid    The ID number of the license you want to cancel (see the *Package List* endpoint).
func (c *Client) CancelLicense(licenseid int) error {
	if licenseid < 1 {
		return fmt.Errorf("%w: bad license ID", ErrInvalidArgument)
	}

	args := url.Values{}
	args.Add("id", strconv.FormatInt(int64(licenseid), 32))
	req := NewRequest(c, "cancel", &args)
	_, err := req.Dispatch()

	return err
}

// SuspendLicense suspends an active license and suspends recurring billing (if any).
//   licenseid    The ID number of the license you want to suspend (see the *Package List* endpoint).
func (c *Client) SuspendLicense(licenseid int) error {
	if licenseid < 1 {
		return fmt.Errorf("%w: bad license ID", ErrInvalidArgument)
	}

	args := url.Values{}
	args.Add("id", strconv.FormatInt(int64(licenseid), 32))
	req := NewRequest(c, "suspend", &args)
	_, err := req.Dispatch()

	return err
}

// UnsuspendLicense unsuspends an active license and resumes recurring billing (if any).
//   licenseid    The ID number of the license you want to unsuspend (see the *Package List* endpoint).
func (c *Client) UnsuspendLicense(licenseid int) error {
	if licenseid < 1 {
		return fmt.Errorf("%w: bad license ID", ErrInvalidArgument)
	}

	args := url.Values{}
	args.Add("id", strconv.FormatInt(int64(licenseid), 32))
	req := NewRequest(c, "unsuspend", &args)
	_, err := req.Dispatch()

	return err
}

// ListLicensesByFilter lists the licenses on the reseller account.
//   idFilter           A slice of license IDs to look up, or nil
//   customidentFilter  A slice of custom identifiers (the `customident` parameter to OrderLicense) to look up, or nil
func (c *Client) ListLicensesByFilter(idFilter []int, customIdentFilter []string) (*response.LicenseList, error) {
	args := url.Values{}

	if len(idFilter) > 0 {
		f := make([]string, 0, len(idFilter))
		for _, v := range idFilter {
			f = append(f, strconv.FormatInt(int64(v), 10))
		}

		args.Add("filter", strings.Join(f, ","))
	}

	if len(customIdentFilter) > 0 {
		args.Add("filtercustomid", strings.Join(customIdentFilter, "|"))
	}

	req := NewRequest(c, "licenselist", &args)
	resp, err := req.Dispatch()
	if err != nil {
		return nil, err
	}

	r := &response.LicenseList{}
	return r, resp.UnmarshalData(r)
}

// ListLicenses lists the licenses on the reseller account.
func (c *Client) ListLicenses() (*response.LicenseList, error) {
	return c.ListLicensesByFilter(nil, nil)
}

// ListLicensesByID lists the licenses on the reseller account.
//   id           Zero or more license IDs to look up
func (c *Client) ListLicensesByID(id ...int) (*response.LicenseList, error) {
	return c.ListLicensesByFilter(id, nil)
}

// ListLicensesByCustomIdent lists the licenses on the reseller account.
//   customident  Zero or more custom identifiers (the `customident` parameter to OrderLicense) to look up, or nil
func (c *Client) ListLicensesByCustomIdent(customident ...string) (*response.LicenseList, error) {
	return c.ListLicensesByFilter(nil, customident)
}

// ListPackages lists all packages available for ordering.
func (c *Client) ListPackages() (*response.PackageList, error) {
	req := NewRequest(c, "packagelist", &url.Values{})
	resp, err := req.Dispatch()
	if err != nil {
		return nil, err
	}

	r := &response.PackageList{}
	return r, resp.UnmarshalData(r)
}

// GetBalance retrieve the balance on your account.
func (c *Client) GetBalance() (response.CurrencyInCents, error) {
	req := NewRequest(c, "balance", &url.Values{})
	resp, err := req.Dispatch()
	if err != nil {
		return 0, err
	}

	r := &struct{
		Balance response.CurrencyInCents `json:"balance"`
	}{}
	if err := resp.UnmarshalData(r); err != nil {
		return 0, err
	}

	return r.Balance, nil
}