//
// Centova Reseller API - https://centova.com
// Copyright 2021, Centova Technologies Inc.
//
// This library is a Go implementation of a Centova Technologies Reseller API client.
//

package resellerapi

import (
	"crypto/sha256"
	"encoding/json"
	"fmt"
	"net/http"
	"net/url"
	"time"
)

// Request represents an API request to be sent to the reseller API
type Request struct {
	// the Client making the request
	client  *Client
	// the command for the request
	command string
	// the arguments for the request
	args    *url.Values
}

// NewRequest creates a new request for the specified command and arguments.
func NewRequest(c *Client, command string, args *url.Values) *Request {
	args.Set("username", c.username)
	return &Request{
		client:  c,
		command: command,
		args:    args,
	}
}

// hashSHA256 is a helper function to generate the SHA256 hash for a string and return it as a hex string.
func hashSHA256(s string) string {
	hash := sha256.New()
	hash.Write([]byte(s))
	return fmt.Sprintf("%x", hash.Sum(nil))
}

// Dispatch prepares and sends the request, and returns the API response.
func (r *Request) Dispatch() (*APIResponse, error) {
	uri := fmt.Sprintf("%s/%s?%s&time=%d", r.client.endpointBase, r.command, r.args.Encode(), time.Now().Unix())

	token := hashSHA256(r.client.username + r.client.apikey)
	signature := hashSHA256(token + uri)

	url := fmt.Sprintf("https://%s%s&sig=%s", r.client.domain, uri, signature)
	res, err := http.Get(url)
	if err != nil {
		return nil, err
	}

	response := &APIResponse{}

	d := json.NewDecoder(res.Body)
	if err := d.Decode(&response); err != nil {
		return nil, fmt.Errorf("failed to decode API response: %w", err)
	}

	if res.StatusCode < 200 || res.StatusCode > 299 {
		message := response.Message
		if message == "" {
			message = res.Status
		}
		return nil, &APIRequestError{code: res.StatusCode, message: message, response: response}
	}

	return response, nil
}
