//
// Centova Reseller API - https://centova.com
// Copyright 2021, Centova Technologies Inc.
//
// This library is a Go implementation of a Centova Technologies Reseller API client.
//

package response

import (
	"fmt"
	"strconv"
	"strings"
)

// CurrencyInCents represents currency values returned by the API in terms of total number of cents.
// For example, $10.32 would be represented as 1032.
type CurrencyInCents int

// ToDollars returns a floating-point representation of the currency value.
func (c CurrencyInCents) ToDollars() float64 {
	return float64(c) / 100
}

// String returns a formatted string representation of the currency value.
// For example, 1032 would be returned as "10.32".
func (c CurrencyInCents) String() string {
	return fmt.Sprintf("%d.%.02d", int(c/100), c%100)
}

// FromString sets this currency value from a string representation.
// For example, FromString("10.32") sets the value to 1032.
func (c *CurrencyInCents) FromString(s string) error {
	pieces := strings.Split(s, ".")
	if len(pieces) == 0 || len(pieces) > 2 {
		return strconv.ErrSyntax
	}

	v, err := strconv.ParseInt(pieces[0], 10, 64)
	if err != nil {
		return err
	}

	*c = CurrencyInCents(v) * 100

	if len(pieces) > 1 {
		v, err = strconv.ParseInt(pieces[1], 10, 64)
		if err != nil {
			return err
		}

		if len(pieces[1]) == 1 {
			v *= 10
		}

		*c += CurrencyInCents(v)
	}

	return nil
}

func (c *CurrencyInCents) UnmarshalJSON(b []byte) error {
	if len(b) >= 2 && b[0] == '"' && b[len(b)-1] == '"' {
		b = b[1 : len(b)-2]
	}
	return c.FromString(string(b))
}

func (c *CurrencyInCents) MarshalJSON() ([]byte, error) {
	return []byte(c.String()), nil
}
