# Centova Cast Reseller API

Copyright 2021, Centova Technologies Inc.  
https://centova.com


## Introduction

This file contains information needed to integrate the Centova Technologies
reseller API system into your own billing system to automatically submit 
orders through your reseller account.


## Limits

Please be reasonable in your deployment of any API client solution.  When 
possible, you should cache any data that is unlikely to change often (such 
as the package and order list) to avoid hammering the Centova Techologies
licensing servers with requests.

Centova Technologies reserves the right to reject traffic from networks that
submit requests an excessive rate. We do not precisely define "excessive",
however a sustained rate exceeding one request per second for any substantial
length of time would be inappropriate.


## Generating an API Key

An API key is required to access the Reseller API. To generate your API key,
log in to the Centova Technologies client area at https://centova.com and
select *Reseller Settings* under the *Clients* menu.

Near the bottom of the page, under the *Reseller API* section, clikc the
*Create API key* button to create your API key. If you have previously
generated an API key, you may click the *Reset API key* button to create a new
key.

Be sure to copy down the generated key, as it is only shown once and cannot be
retrieved again.


## Making an API Request

API requests are performed through simple GET or POST requests to a Reseller
API endpoint. The URL for the live API is:

    https://centova.com/cp/api/v1/reseller/

For the testing API, use:

    https://centova.com/cp/api/v1/resellertest/

Each API request must include, at minimum, the following query variables in the
URL:

 - `username`  your account username
 - `time`      the current time as a UNIX timestamp
 - `sig`       the signature for the request

For example, to query the testing API for the account balance for the user `bob`,
you might make a GET request to:

    https://centova.com/cp/api/v1/resellertest/balance?username=bob&time=1623905353&sig=abcd1234

The following sections describe the available endpoints and the instructions for
signing a request.


## Signing a Request

You must sign all API requests to confirm your identity. To sign a request:

1. Prepare the URI for your API request as described in the previous section.
   For example, if your username is `bob`, and you want to query the testing
   API for your current balance, your URI would be:
   
        /cp/api/v1/resellertest/balance?username=bob

   Do not include `https://centova.com` in the URI. If the API endpoint accepts
   any other query variables that you want to use, add them to the URI now.
   

2. Add a query variable for the current time, as a UNIX timestamp, to the URI:

        /cp/api/v1/resellertest/balance?username=bob&time=1623905353


3. Calculate your authentication token by concatenating your username and your
   API key, and take the SHA256 hash of the resulting string:
   
        sha256(username + apikey)

   For example, if your username is `bob` and your API key is `abcd1234`, your
   authentication token would be generated as:
   
        sha256("bobabcd1234")

   Which yields:

        cc353c1917f592ada5d8aa11ebf3617c8a139d99a8fc9349e6927246e3ebbca4


4. Calculate the signature for the request by concatenating your authentication
   token and your URI, and take the SHA256 hash of hte resulting string:
   
        sha256(token + uri)

   For  example, using the values generated above (linebreak added for clarity):
   
        SHA256(cc353c1917f592ada5d8aa11ebf3617c8a139d99a8fc9349e6927246e3ebbca4/
            cp/api/v1/resellertest/balance?username=bob&time=1623905353)

    Which yields your signature:

        198a6b697744c417d5cfd5b8436a3e1390232809b3edc16e72b447c52c58c976


5. Add a query variable for the signature to the *end* of the URI (linebreak
   added for clarity):

        /cp/api/v1/resellertest/balance?username=bob&time=1623905353&sig=198a6
            b697744c417d5cfd5b8436a3e1390232809b3edc16e72b447c52c58c976
   
   This is your completed, signed request URI. Just prepend `https://centova.com`
   to it and use it to dispatch your GET or POST request.


## API Responses

The Reseller API generates simple responses in JSON format with the following
structure:

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "It worked!",        // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
        // ...                      // any data generated by the request
    }
}
```


## Testing API

Our testing API provides a simulated environment in which you can test your
Reseller API integration code without the risk of impacting your real purchases
or balance.

To use the testing API, simply use `/cp/api/v1/resellertest/` as your URI base
instead of `/cp/api/v1/reseller/`.

Your existing username and API key will work with the testing API, and your
account has an initial test balance of $25000 (no real-world value) which you
can use for test purchases.

Licenses created through the test system will not be functional (i.e., they
cannot be used with our software).


## API Endpoints

The available endpoints are described below. Note that each request must be
signed as described in the *Signing a Request* section above.


### Place an Order

Place an order for the specified package.

#### Request

    POST /cp/api/v1/reseller/order?username=USERNAME&package=PACKAGE&period=PERIOD&domain=DOMAIN
        &installation=INST&email=EMAIL
    
 - `USERNAME`   Your centova.com username
 - `PACKAGE`    The ID number of the package to order (see the *Package List*
                    endpoint)
 - `PERIOD`     The renewal period for the license:
    - for leased licenses: `monthly`, `quarterly`, `biannually`, or `annually`
    - for owned licenses, leave this empty
 - `DOMAIN`      The domain name for the new license
 - `EMAIL`       The end-user's email address (used to email license key to user)
 - `INST`        `1` if installation service is being ordered, otherwise `0`

Other optional arguments include:

 - `&customident=IDENT` Any custom identifier that you want to associate with this
                            license
 - `&coupon=COUPON`    A coupon code to apply to this purchase (will negate any
                            reseller discount)

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "OK",                // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
        "license_id": 1,            // the ID number for the new license    
        "invoice_id": 1,            // the invoice ID for this order
        "amount_recurring": "1.23", // the recurring amount (in USD) to be charged 
        "amount_invoice": "1.23"    // the amount (in USD) of the first invoice
    }
}
```


### Cancel a License

Cancel an active license and terminate recurring billing (if any).

#### Request

    POST /cp/api/v1/reseller/cancel?username=USERNAME&id=LICENSEID

 - `USERNAME`   Your centova.com username
 - `LICENSEID`  The ID number of the license you want to cancel (see the *Package
                    List* endpoint).

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "x cancelled",       // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
    }
}
```


### Suspend a License

Suspend an active license and suspends recurring billing (if any).

#### Request

    POST /cp/api/v1/reseller/suspend?username=USERNAME&id=ID

 - `USERNAME`   Your centova.com username
 - `LICENSEID`  The ID number of the license you want to suspend (see the
                    *Package List* endpoint).

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "x suspended",       // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
    }
}
```


### Unsuspend a License

Unsuspend a suspended license and resume recurring billing (if any).

#### Request

    POST /cp/api/v1/reseller/unsuspend?username=USERNAME&id=ID

 - `USERNAME`   Your centova.com username
 - `LICENSEID`  The ID number of the license you want to unsuspend (see the
                    *Package List* endpoint).

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "x unsuspended",     // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
    }
}
```


### License List

List licenses on your account.

#### Request

    GET /cp/api/v1/reseller/licenselist?username=USERNAME

 - `USERNAME`   Your centova.com username

Other optional arguments include:

 - `&filter=FILTERID` where `FILTERID` represents one or more license ID numbers
        (comma-separated) to search for.
 - `&filtercustomid=IDENT` where `IDENT` represents the `IDENT` value used when
        the order was placed (see the *Place an order* endpoint).

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "OK",                // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
        "licenses": [               // a list of matching licenses in your account
            {
                "license_id": 1,                            // license ID number
                "client_email": "example@example.com",      // client email address
                "identifier": "my-custom-identifier",       // IDENT passed when creating
                "package_id": 42,                           // package ID (see "list packages")
                "package_name": "Widget License",           // package name
                "status": "Active",                         // license status
                "display_name": "License for example.com",  // human-friendly license name
                "creation_date": "2021-01-02 03:04:05",     // date order was created
                "billing_period": "monthly",                // billing period, if leased
                "billing_amount": "1.23",                   // recurring billing amount, if leased
                "billing_next": "2021-02-03",               // next billing date, if leased
                "license_key": "abcd1234",                  // product license key
                "licensed_domain": "example.com",           // license issued domain name
                "licensed_ip": "10.11.12.23",               // license issued IP address
                "licensed_mac": "01:02:03:04:05:06",        // license issued MAC address
                "licensed_path": "/foo/bar/baz",            // license issued path
                "issued": 1,                                // 1 if issued, 0 if not
                "issue_date": "2021-01-02 04:05:06",        // date on which license was issued
                "expiration": "",                           // expiration date for the license                
            },
            // ...
        ]
    }
}
```


### Package List

List all packages available for ordering.

#### Request

    GET /cp/api/v1/reseller/packagelist?username=USERNAME

 - `USERNAME`   Your centova.com username

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "OK",                // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
        "packages": [               // a list of packages available for ordering
            {
                "package_id": 1,                    // package ID number
                "package_name": "Widget License",   // name of the package
                "price_monthly": "10.99",           // monthly price if leased, or the owned price if owned
                "price_quarterly": "29.99",         // quarterly price, if available
                "price_biannually": "59.99",        // biannual price, if available
                "price_annually": "119.99",         // annual price, if available
                "price_setup": "29.99",             // setup fee, if any
                "recurring": 1,                     // 1 if leased (with recurring payments), 0 if owned       
            },
            // ...
        ]
    }
}
```


### Balance Check

Retrieve the balance on your account.

#### Request

    GET /cp/api/v1/reseller/balance?username=USERNAME

 - `USERNAME`   Your centova.com username

#### Response

```json
{
    "result":  "success",           // either `success` or `error`
    "code":    200,                 // the HTTP response code for the request
    "message": "OK",                // a plain-English description of the result
    "data": {                       // the result data for the request
        "mode": "live",             // `live` or `test`, depending on the endpoint
        "balance": "25000.00",      // the available balance (in USD) in your account
    }
}
```

