<?php
/*
 * Centova Reseller API - https://centova.com
 * Copyright 2008-2021, Centova Technologies Inc.
 *
 ******************************************************************************
 *
 * This library is a PHP implementation of a Centova Technologies Reseller API
 * client.
 *
 */
namespace Centova\Reseller;

/**
 * Class APIClient
 *
 * @package Centova\Reseller
 */
class APIClient {

	/** @var bool test mode enabled or disabled */
	private $testmode = false;

	/** @var string the base URL to the API endpoint */
	private $api_base_url = '';

	/** @var string Centova Technologies' secure server domain - shouldn't need to modify this */
	private $centovacom_host = 'centova.com';
	/** @var string reseller API URI - shouldn't need to modify this */
	private $base_uri = '/cp/api/v1/';

	/** @var string the endpoint base for the live API */
	private $endpoint_base_live = 'reseller';
	/** @var string the endpoint base for the testing API */
	private $endpoint_base_test = 'resellertest';

	/** @var string the API username */
	private $username = '';
	/** @var string the API key */
	private $key = '';
	/** @var bool use SSL encryption */
	private $ssl = true;

	/**
	 * Constructor.
	 *
	 * @param string $username Your centova.com client account username
	 * @param string $key Your centova.com reseller API key
	 * @param bool $testmode TRUE if the test server should be used, FALSE
	 *                       if the live server should be used
	 */
	public function __construct($username,$key,$testmode=false) {
		$this->testmode = $testmode;
		$this->username = $username;
		$this->key = $key;
		$this->generateBaseURL();
	}

	/**
	 * Places an order for a package.
	 *
	 * @param int         $packageid    The ID number of the package to order (see list_packages())
	 * @param string      $period       The renewal period for the license (monthly, quarterly, biannually, annually,
	 *                                  or NULL for non-recurring (i.e., owned) packages
	 * @param string      $domain       The domain name for the new license
	 * @param string      $email        The end-user's E-mail address (used to E-mail license key to user)
	 * @param bool        $installation True if installation service is being ordered, otherwise false
	 * @param mixed       $customident  Any custom identifier that you want to associate with this license, or null
	 * @param bool|string $coupon       A coupon code to apply to this purchase (will negate any reseller discount) or null
	 *
	 * @return array an array containing the following keys on success:
	 * 	license_id - the ID number for the new license     (1)
	 * 	invoice_id - the invoice ID for this order (1)
	 * 	amount_recurring - the recurring amount (in USD) to be charged  ("1.23")
 	 *  amount_invoice - the amount (in USD) of the first invoice ("1.23")	 
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function orderLicense($packageid, $period, $domain, $email, $installation = false, $customident = null, $coupon = null) {
		$periods = array(NULL,'monthly','quarterly','biannually','annually');

		if (!is_numeric($packageid)) throw new APIClientArgumentError('Invalid value for package ID');
		if (!in_array($period,$periods)) throw new APIClientArgumentError('Invalid value for billing period');
		if (!is_bool($installation)) throw new APIClientArgumentError('Invalid value for installation');
		if (!preg_match('/^.+\..+$/',$domain)) throw new APIClientArgumentError('Invalid value for domain name');
		if (!preg_match('/^.+\@.+\..+$/',$email)) throw new APIClientArgumentError('Invalid value for E-mail address');
		
		$args = array(
			'package'=>(int) $packageid,
			'period'=>$period,
			'domain'=>$domain,
			'installation'=>$installation ? 1 : 0,
			'customid'=>$customident,
			'email'=>$email,
			'coupon'=>$coupon
		);

		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'order',$args);
		$response = $request->dispatch();
		
		return $response->getData();
	}

	/**
	 * Cancels an active license and terminates recurring billing (if any).
	 *
	 * @param int $licenseid The ID number of the license you want to cancel (see list_licenses()).
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function cancelLicense($licenseid) {
		if (!is_numeric($licenseid)) throw new APIClientArgumentError('Invalid value for license ID '.$licenseid);

		$args = array('id'=>(int) $licenseid);

		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'cancel',$args);
		$request->dispatch();
	}

	/**
	 * Suspends an active license and suspends recurring billing (if any).
	 *
	 * @param int $licenseid The ID number of the license you want to suspend (see list_licenses()).
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function suspendLicense($licenseid) {
		if (!is_numeric($licenseid)) throw new APIClientArgumentError('Invalid value for license ID');

		$args = array('id'=>(int) $licenseid);

		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'suspend',$args);
		$request->dispatch();
	}
	
	/**
	 * Unsuspends a suspended license and resumes recurring billing (if any).
	 *
	 * @param int $licenseid The ID number of the license you want to resume (see list_licenses()).
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function unsuspendLicense($licenseid) {
		if (!is_numeric($licenseid)) throw new APIClientArgumentError('Invalid value for license ID');

		$args = array('id'=>(int) $licenseid);

		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'unsuspend',$args);
		$request->dispatch();
	}
	
	/**
	 * Lists all licenses on your account.
	 *
	 * @param mixed $filter_by_id An integer or array of integers representing the ID number(s) of
	 *                            the license(s) to return, or null for all.
	 * @param mixed $filter_by_customident A string or array of strings representing the
	 *                                     "customident" values of the license(s) to return, or
	 *                                     null for all.  (The customident value is specified when
	 *                                     calling the order() method).
	 *
	 * @return array a list of licenses; each license is a subarray with the following keys:
	 * 	license_id - license ID number (1)
	 * 	client_email - client email address ("example@example.com")
	 * 	identifier - IDENT passed when creating ("my-custom-identifier")
	 * 	package_id - package ID (see "list packages") (42)
	 * 	package_name - package name ("Widget License")
	 * 	status - license status ("Active")
	 * 	display_name - human-friendly license name ("License for example.com")
	 * 	creation_date - date order was created ("2021-01-02 03:04:05")
	 * 	billing_period - billing period, if leased ("monthly")
	 * 	billing_amount - recurring billing amount, if leased ("1.23")
	 * 	billing_next - next billing date, if leased ("2021-02-03")
	 * 	license_key - product license key ("abcd1234")
	 * 	licensed_domain - license issued domain name ("example.com")
	 * 	licensed_ip - license issued IP address ("10.11.12.23")
	 * 	licensed_mac - license issued MAC address ("01:02:03:04:05:06")
	 * 	licensed_path - license issued path ("/foo/bar/baz")
	 * 	issued - 1 if issued, 0 if not (1)
	 * 	issue_date - date on which license was issued ("2021-01-02 04:05:06")
	 * 	expiration - expiration date for the license                
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function listLicenses($filter_by_id = null, $filter_by_customident = null) {
		$args = array();
		
		if (!empty($filter_by_id)) {
			if (!is_array($filter_by_id)) {
				$filter_by_id = array($filter_by_id);
			}

			array_walk($filter_by_id,function($id){
				if (!is_numeric($id)) throw new APIClientArgumentError(sprintf('Invalid value %s in ID filter list',$id));
			});

			$args['filter'] = implode(',',$filter_by_id);

		} elseif (!empty($filter_by_customident)) {
			if (!is_array($filter_by_customident)) {
				$filter_by_customident = array($filter_by_customident);
			}

			array_walk($filter_by_customident,function(&$customident){
				if (!strlen($customident)) throw new APIClientArgumentError('Empty value in custom-ident filter list');
				$customident = str_replace('|','',$customident);
			});
			$args['filtercustomid'] = implode('|',$filter_by_customident);
		}


		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'licenselist',$args);
		$response = $request->dispatch();
		
		$data = $response->getData();
		return $data['licenses'];
	}
	
	/**
	 * Lists all packages available for ordering.
	 *
	 * @return array the package list; each package is a subarray with the following keys:
	 * 	package_id - package ID number (1)
	 * 	package_name - name of the package ("Widget License")
	 * 	price_monthly - monthly price if leased, or the owned price if owned ("10.99")
	 * 	price_quarterly - quarterly price, if available ("29.99")
	 * 	price_biannually - biannual price, if available ("59.99")
	 * 	price_annually - annual price, if available ("119.99")
	 * 	price_setup - setup fee, if any ("29.99")
	 * 	recurring - 1 if leased (with recurring payments), 0 if owned        (1)
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function listPackages() {

		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'packagelist');
		$response = $request->dispatch();
		
		$data = $response->getData();
		return $data['packages'];
	}
	
	/**
	 * Retrieves the balance on your account.  A positive value indicates a credit on your account;
	 * a negative value indicates an amount owing.
	 *
	 * @return string the current account balance in USD
	 *
	 * @throws APIClientArgumentError if a value passed as an argument is not valid
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function getBalance() {
		$request = new APIRequest($this->api_base_url, $this->username, $this->key,'balance');
		$response = $request->dispatch();

		$data = $response->getData();
		return $data['balance'];
	}

	/**
	 * Generate the base URL for API requests
	 */
	private function generateBaseURL() {
		$this->api_base_url = 'http'. ($this->ssl ? 's' : '') . '://' . $this->centovacom_host . $this->base_uri . ($this->testmode ? $this->endpoint_base_test : $this->endpoint_base_live);
	}

	/**
	 * For maintainer use.
	 *
	 * @param string $host
	 * @param string $uri
	 * @param string $live
	 * @param string $test
	 * @param bool $ssl;
	 */
	public function setAlternateEndpoint($host, $uri, $live, $test, $ssl) {
		$this->centovacom_host = $host;
		$this->centovacom_uri = $uri;
		$this->endpoint_base_live = $live;
		$this->endpoint__base_test = $test;
		$this->ssl = $ssl;
		$this->generateBaseURL();
	}

}
