<?php
/*
 * Centova Reseller API - https://centova.com
 * Copyright 2008-2021, Centova Technologies Inc.
 *
 *****************************************************************************
 *
 * This library is a PHP implementation of a Centova Technologies Reseller API
 * client.
 *
 *
 */

namespace Centova\Reseller;

/**
 * Class APIRequest
 *
 * @package Centova\Reseller
 */
class APIRequest {

	const MODE_UNSUPPORTED = 0;
	const MODE_POST_FGC    = 1;
	const MODE_POST_FSO    = 2;
	const MODE_GET_FGC     = 3;
	/** @var string query string for the API request */
	private $querystr = '';
	/** @var int HTTP request mode */
	private $mode = 0;
	
	private $base_url = '';
	private $username = '';
	private $key = '';

	/**
	 * Dispatches an API call to the server.
	 *
	 * @param string $base_url The base URL for the API server endpoint
	 * @param string $username The reseller API username
	 * @param string $key      The reseller API key
	 * @param string $command  The command to execute
	 * @param array  $args     The arguments to pass to the server
	 *
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 */
	public function __construct($base_url, $username, $key, $command, $args = null) {
		$this->base_url = $base_url . '/' . $command;
		$this->username = $username;
		$this->key = $key;

		if (!is_array($args)) {
			$args = array();
		}
		$args['username'] = $this->username;

		$this->querystr = http_build_query($args);

		// best case is an HTTP POST performed via file_get_contents, but this requires
		// fopen wrappers and context support which is only available in 5.0.0+
		if (ini_get('allow_url_fopen') && !version_compare(PHP_VERSION, '5.0.0', '<')) {
			$this->mode = self::MODE_POST_FGC;

			// next best case is an HTTP POST provide via raw socket functions, but
			// these require sockets support
		} else if (function_exists('fsockopen')) {
			$this->mode = self::MODE_POST_FSO;

			// last resort is an HTTP GET performed via file_get_contents
		} else if (ini_get('allow_url_fopen')) {
			$this->mode = self::MODE_GET_FGC;

			// fail if none of the above methods were successful
		} else {
			throw new APIClientEnvironmentError('Your installation of PHP is not built/configured with any means to connect to remote servers via HTTPS.');
		}
	}

	/**
	 * Dispatches a request to the API server
	 *
	 * @return APIResponse an object representing the server's response to the request
	 *
	 * @throws APIClientEnvironmentError if PHP is not capable of dispatching the request
	 * @throws APIClientRequestError if an HTTP error occurs during the API request
	 * @throws APIClientServerError if the API server returns an error
	 */
	public function dispatch() {
		$urlinfo = parse_url($this->base_url);
		$uri = $urlinfo['path'] . '?' . $this->querystr . '&time='. time();
		
		$token = hash('sha256',$this->username . $this->key);
		$signature = hash('sha256',$token . $uri);
		
		$uri .= '&sig='.$signature;
		
		$signed_url = $urlinfo['scheme'] . '://' . $urlinfo['host'] . $uri;

		switch ($this->mode) {
			case self::MODE_POST_FGC:
				$response = $this->httpPostFGC($signed_url);
				break;
			case self::MODE_POST_FSO:
				$response = $this->httpPostFSO($signed_url);
				break;
			case self::MODE_GET_FGC:
				$response = file_get_contents($signed_url);
				break;
			default:
				throw new APIClientEnvironmentError();
		}

		return new APIResponse($response);
	}

	/**
	 * Dispatch a POST request via file_get_contents().
	 *
	 * @param string $url      the URL
	 * @param string $querystr the query string
	 *
	 * @return string the response body
	 * @throws APIClientRequestError on error
	 */
	private function httpPostFGC($url, $body = null) {
		$contextopts = array(
			'http' => array(
				'method'  => 'POST',
				'header'  => 'Content-type: application/x-www-form-urlencoded',
				'content' => $body,
				'ignore_errors' => true,
			)
		);
		$context = stream_context_create($contextopts);
		
		ob_start();
		$fp = fopen($url,'r',false,$context);
		$meta = stream_get_meta_data($fp);
		$result = stream_get_contents($fp);
		fclose($fp);
		ob_end_clean();

		$status = $meta['wrapper_data'][0];
		list(,$code,$message) = explode(' ',$status,3);

		if ($code < 200 || $code > 299) {
			throw new APIClientRequestError($message,$code);
		}

		return $result;
	}

	/**
	 * Dispatch a POST request via fsockopen().
	 *
	 * @param string $host     the server hostname
	 * @param string $uri      the URI
	 * @param string $querystr the query string
	 * @param bool   $ssl      true for SSL, false for plaintext http
	 *
	 * @return string the response body
	 * @throws APIClientRequestError on error
	 */
	private function httpPostFSO($url) {
		$urlinfo = parse_url($url);
		
		$tls = $urlinfo['scheme'] == 'https';
		$port = $tls ? 443 : 80;
		$host = $tls ? 'ssl://' . $urlinfo['host'] : $urlinfo['host']; 

		$fp = fsockopen($host, $port, $errno, $errstr, 10);
		if (!$fp) {
			throw new APIClientRequestError('Error connecting to API server: ' . $errstr);
		}

		if (function_exists('stream_set_timeout')) {
			stream_set_timeout($fp, 30);
		}
		$uri = $urlinfo['path'] . '?' . $urlinfo['query'];
		$host = $urlinfo['host'];
		
		fwrite($fp, "GET {$uri} HTTP/1.1\nHost: {$host}\nConnection: close\n\n");

		$response = '';
		while (!@feof($fp)) {
			$response .= fgets($fp, 10240);
			if (function_exists('socket_get_status')) {
				$meta = socket_get_status($fp);
				if ($meta['timed_out']) {
					break;
				}
			}
		}
		@fclose($fp);

		list($headers, $response) = explode("\r\n\r\n", $response, 2);

		$headers = trim($headers);
		list($status,) = explode("\n", $headers, 2);
		list(,$code,$message) = explode(' ',$status,3);

		if ($code < 200 || $code > 299) {
			throw new APIClientRequestError($message,$code);
		}

		// decode HTTP/1.1 chunked transfer encoding
		if (preg_match('/\nTransfer-Encoding: (.*?)\r?\n/i', $headers, $matches)) {
			if ($matches[1] == 'chunked') {
				$remaining = $response;
				$response = "";

				while ($remaining) {
					$hexlen = strpos($remaining, "\r");
					$chunksize = substr($remaining, 0, $hexlen);
					$argstart = strpos($chunksize, ';');
					if ($argstart !== false) {
						$chunksize = substr($chunksize, 0, $argstart);
					}
					$chunksize = (int)@hexdec($chunksize);

					$response .= substr($remaining, $hexlen + 2, $chunksize);
					$remaining = substr($remaining, $hexlen + 2 + $chunksize + 2);

					if (!$chunksize) {
						// either we're done, or something's borked... exit
						$response .= $remaining;
						break;
					}
				}
			}
		}

		return $response;
	}

}