<?php
/*
 * Centova Reseller API - https://centova.com
 * Copyright 2008-2021, Centova Technologies Inc.
 *
 *****************************************************************************
 *
 * This library is a PHP implementation of a Centova Technologies Reseller API
 * client.
 *
 *
 */

namespace Centova\Reseller;

/**
 * Class APIResponse
 *
 * @package Centova\Reseller
 */
class APIResponse {

	/** @var bool debuggin? */
	private static $debug = false;
	/** @var bool true if the request was successful, otherwise false */
	private $successful = false;
	/** @var int the HTTP response code */
	private $code = 0;
	/** @var string the response message received from the server, if any */
	private $message = '';
	/** @var array the response data received from the server, if any */
	private $data = array();

	/**
	 * Constructor
	 *
	 * @param string $response a response from the API server
	 *
	 * @throws APIClientServerError
	 */
	public function __construct($response) {
		$this->parse($response);
	}

	/**
	 * Parse a response from the API srver.
	 *
	 * @param string $response A response from the API server.
	 *
	 * @throws APIClientServerError
	 */
	private function parse($response) {
		if (self::$debug) {
			echo "=== response ===\n$response\n===\n";
		}
		if (preg_match('/>(SQL Error:|(Fatal|Parse) error|Warning)</', $response)) {
			throw new APIClientServerError('Service temporarily unavailable');
		}

		$payload = json_decode($response,true);
		if ($payload === false) {
			throw new APIClientServerError('JSON decode failed: ' . json_last_error_msg());
		}

		$this->code = $payload['code'];
		$this->successful = $payload['result'] == 'success';
		$this->message = $payload['message'];
		$this->data = $payload['data'];
	}

	public static function setDebug($debug = true) {
		self::$debug = $debug;
	}

	/**
	 * Gets the response disposition from the server.
	 * 
	 * @return bool true on success, false on error
	 */
	public function getSuccessful() {
		return $this->successful;
	}
	
	/**
	 * Gets the response message returned by the server.
	 *
	 * @return string the message
	 */
	public function getMessage() {
		return $this->message;
	}

	/**
	 * Gets the HTTP response code returned by the server.
	 * 
	 * @return int the HTTP response code 
	 */
	public function getCode() {
		return $this->code;
	}

	/**
	 * Gets the data returned by the server.
	 *
	 * @return array an array of items
	 */
	public function getData() {
		return $this->data;
	}

} 